use std::collections::HashMap;

use crate::enums::{AudioMixOption, AudioSourceType, ExternalPortType};

pub type AudioChannel = ClassicAudioChannel;
pub type AudioMasterChannel = ClassicAudioMasterChannel;
pub type AtemAudioState = AtemClassicAudioState;

#[derive(Getters, new)]
pub struct ClassicAudioChannel {
    source_type: AudioSourceType,
    pub port_type: ExternalPortType,
    pub mix_option: AudioMixOption,
    pub gain: f64,
    pub balance: f64,

    supports_rca_to_xlr_enabled: bool,
    pub rca_to_xlr_enabled: bool,
}

#[derive(Getters, new)]
pub struct ClassicAudioMasterChannel {
    pub gain: f64,
    pub balance: f64,
    pub follow_fade_to_black: bool,
}

#[derive(Getters, new)]
pub struct ClassicAudioMonitorChannel {
    pub enabled: bool,
    pub gain: f64,
    pub mute: bool,
    pub solo: bool,
    pub solo_source: f64,
    pub dim: bool,
    pub dim_level: f64,
}

#[derive(Getters, new)]
pub struct ClassicAudioHeadphoneOutputChannel {
    pub gain: f64,
    pub program_out_gain: f64,
    pub sidetone_gain: f64,
    pub talkback_gain: f64,
}

#[derive(Getters, new)]
pub struct AtemClassicAudioState {
    number_of_channels: Option<f64>,
    has_monitor: Option<bool>,
    pub channels: HashMap<f64, ClassicAudioChannel>,
    pub monitor: Option<ClassicAudioMonitorChannel>,
    pub headphones: Option<ClassicAudioHeadphoneOutputChannel>,
    pub master: Option<ClassicAudioMasterChannel>,

    pub audio_follow_video_crossfade_transition_enabled: Option<bool>,
}
