use std::collections::HashMap;

use crate::enums::{ExternalPortType, FairlightAnalogInputLevel, FairlightAudioMixOption, FairlightAudioSourceType, FairlightInputConfiguration, FairlightInputType};

#[derive(Getters, new)]
pub struct FairlightAudioDynamicsState {
    pub make_up_gain: Option<u64>,
    
    pub limiter: Option<FairlightAudioLimiterState>,
    pub compressor: Option<FairlightAudioCompressorState>,
    pub expander: Option<FairlightAudioExpanderState>
}

#[derive(Getters, new)]
pub struct FairlightAudioLimiterState {
    pub limiter_enabled: bool,
    pub threshold: u64,
    pub attack: u64,
    pub hold: u64,
    pub release: u64
}

#[derive(Getters, new)]
pub struct FairlightAudioCompressorState {
    pub compressor_enabled: bool,
    pub threshold: u64,
    pub ratio: u64,
    pub attack: u64,
    pub hold: u64,
    pub release: u64
}

#[derive(Getters, new)]
pub struct FairlightAudioExpanderState {
    pub expander_enabled: bool,
    pub gate_enabled: bool,
    pub threshold: u64,
    pub range: u64,
    pub ratio: u64,
    pub attack: u64,
    pub hold: u64,
    pub release: u64
}

#[derive(Getters, new)]
pub struct FairlightAudioEqualizerBandState {
    pub band_enabled: bool,

    supported_shapes: Vec<u64>, // TODO
    pub shape: u64, // TODO

    supported_frequency_ranges: Vec<u64>, // TODO
    pub frequency_ranges: u64, // TODO

    pub frequency: u64,

    pub gain: u64,
    pub q_factor: u64
}

#[derive(Getters, new)]
pub struct FairlightAudioMasterChannelPropertiesState {
    // Gain in decibel, -Infinity to +6dB
    pub fader_gain: u64,
    pub follow_fade_to_black: bool
}

#[derive(Getters, new)]
pub struct FairlightAudioMasterChannel {
    pub properties: Option<FairlightAudioSourcePropertiesState>,

    pub equalizer: Option<FairlightAudioEqualizerState>,
    pub dynamicss: Option<FairlightAudioDynamicsState>,
}

#[derive(Getters, new)]
pub struct FairlightAudioMonitorChannel {
    pub gain: u64,
    pub input_master_gain: u64,
    pub input_talkback_gain: u64,
    pub input_sidetone_gain: u64
}

#[derive(Getters, new)]
pub struct FairlightAudioSource {
    pub properties: Option<FairlightAudioSourcePropertiesState>,
    pub equalizer: Option<FairlightAudioEqualizerState>,
    pub dynamics: Option<FairlightAudioDynamicsState>
}

#[derive(Getters, new)]
pub struct FairlightAudioEqualizerState {
    pub enabled: bool,
    pub gain: u64,
    bands: Vec<FairlightAudioEqualizerBandState>
}

#[derive(Getters, new)]
pub struct FairlightAudioSourcePropertiesState {
    source_type: FairlightAudioSourceType,

    max_frames_delay: u64,
    pub frames_delay: u64,

    has_stereo_simulation: bool,
    pub stereo_simulation: u64,

    pub gain: u64,
    pub balance: u64,
    pub fader_gain: u64,

    supported_mix_options: Vec<FairlightAudioMixOption>,
    pub mix_option: FairlightAudioMixOption
}

#[derive(Getters, new)]
pub struct FairlightAudioInput {
    pub properties: Option<FairlightAudioInputProperties>,
    pub sources: HashMap<String, FairlightAudioSource>
}

#[derive(Getters, new)]
pub struct FairlightAudioInputProperties {
    input_type: FairlightInputType,
    external_port_type: ExternalPortType,

    supported_configurations: Vec<FairlightInputConfiguration>,
    pub active_configuration: FairlightInputConfiguration,

    supported_input_levels: Vec<FairlightAnalogInputLevel>,
    pub activeInputLevel: FairlightAnalogInputLevel
}

#[derive(Getters, new)]
pub struct AtemFairlightAudioState {
    pub inputs: HashMap<u64, FairlightAudioInput>,
    pub master: Option<FairlightAudioMasterChannel>,
    pub monitor: Option<FairlightAudioMonitorChannel>,

    pub audio_follow_video_crossfade_transition_enabled: Option<bool>
}
